/*
 * Copyright 2010 Inalogic® Inc.
 *
 * This program is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License, as
 * published by the  Free Software Foundation; either version 2.1 or 3.0
 * of the License.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranties of
 * MERCHANTABILITY, SATISFACTORY QUALITY or FITNESS FOR A PARTICULAR
 * PURPOSE.  See the applicable version of the GNU Lesser General Public
 * License for more details.
 *
 * You should have received a copy of both the GNU Lesser General Public
 * License along with this program. If not, see <http://www.gnu.org/licenses/>
 *
 * Authored by: Jay Taoko <jaytaoko@inalogic.com>
 *
 */


#ifndef FOLDERTREEITEM_H
#define FOLDERTREEITEM_H

#include <string>
#include <boost/filesystem.hpp>

#include "TreeControl.h"

namespace nux
{

  class FolderTreeItem: public TreeItem
  {
    //NODE_XML_CLASS_MEMBER(FolderTreeItem);
  public:
    FolderTreeItem (const TCHAR *name, const TCHAR *Path, bool Searchable = true, NodeParameterType type = NODE_TYPE_STATICTEXT);
    virtual ~FolderTreeItem();

    virtual void DrawProperty(GraphicsEngine& GfxContext,
                              TableCtrl *table,
                              bool force_draw,
                              Geometry geo,
                              const BasePainter &Painter,
                              RowHeader *row,
                              const std::vector<ColumnHeader>& column_vector,
                              Color ItemBackgroundColor = Color (0x0) );

    std::string GetPathName()
    {
      return m_Path.string();
    }
    std::string GetFilename()
    {
      return m_Path.filename();
    }

    void SetSynchronize (bool b)
    {
      m_Synchronize = b;
    }
    bool IsSynchronized()
    {
      return m_Synchronize;
    }

    void SetOpenIcon (const BaseTexture* Texture)
    {
      FolderOpenIcon = Texture->Clone ();
    }
    void SetCloseIcon (const BaseTexture* Texture)
    {
      FolderClosedIcon = Texture->Clone ();
    }

    bool IsSearchable()
    {
      return m_Searchable;
    }
  private:
    BaseTexture* FolderOpenIcon;
    BaseTexture* FolderClosedIcon;

    boost::filesystem::path m_Path;
    NString m_FolderName;
    bool m_Synchronize; // synchronize with windows explorer
    // True if this is a real directory that can be explorer.
    // False if this is just a decoration item.
    // Non searchable folders must always be at the top of the tree;
    bool m_Searchable;
  };

}

#endif // FOLDERTREEITEM_H
